"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloseServerWhenTestsComplete = void 0;
/** Closes the server when all the tests complete */
class CloseServerWhenTestsComplete {
    constructor() {
        this.totalTests = 0;
        this.testsCompleted = 0;
    }
    initialise(eventEmitter) {
        eventEmitter.on("callServerStarted", ({ callServer }) => {
            this.callHandlingServerStarted(callServer);
            callServer.on("testStarted", ({ testSession }) => {
                this.testStarted();
                testSession.callFlowSession.on("allPromptsMatched", this.testCompleted.bind(this));
                testSession.callFlowSession.on("timeoutWaitingForMatch", this.testCompleted.bind(this));
            });
        });
    }
    callHandlingServerStarted(callServer) {
        this.server = callServer;
    }
    testStarted() {
        this.totalTests++;
    }
    testCompleted() {
        this.testsCompleted++;
        const allTestsCompleted = this.testsCompleted >= this.totalTests;
        if (this.server && allTestsCompleted) {
            this.server.stop();
        }
    }
}
exports.CloseServerWhenTestsComplete = CloseServerWhenTestsComplete;
