"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const CloseServerWhenTestsComplete_1 = require("./CloseServerWhenTestsComplete");
const Emitter_1 = require("../Emitter");
class StubCallServer extends Emitter_1.TypedEmitter {
    listen() {
        return Promise.resolve(undefined);
    }
    stop() {
        // Intentionally empty
    }
}
class StubCallFlowSession extends Emitter_1.TypedEmitter {
}
describe("Close server when tests complete", () => {
    let pluginEmitter;
    let callServer;
    beforeEach(() => {
        pluginEmitter = new Emitter_1.TypedEmitter();
        callServer = new StubCallServer();
    });
    test("server closed when all call-flow sessions have matched all their prompts", () => {
        const stopWhenAllTestsComplete = new CloseServerWhenTestsComplete_1.CloseServerWhenTestsComplete();
        stopWhenAllTestsComplete.initialise(pluginEmitter);
        pluginEmitter.emit("callServerStarted", {
            callServer: callServer,
        });
        const callFlowSession = new StubCallFlowSession();
        const testSession = {
            callFlowTestDefinition: undefined,
            call: undefined,
            callFlowSession,
        };
        callServer.emit("testStarted", { testSession });
        jest.spyOn(callServer, "stop");
        callFlowSession.emit("allPromptsMatched", {
            transcription: "",
        });
        expect(callServer.stop).toHaveBeenCalled();
    });
    test("server closed when all call-flow sessions have timed out", () => {
        const stopWhenAllTestsComplete = new CloseServerWhenTestsComplete_1.CloseServerWhenTestsComplete();
        stopWhenAllTestsComplete.initialise(pluginEmitter);
        pluginEmitter.emit("callServerStarted", {
            callServer: callServer,
        });
        const callFlowSession = new StubCallFlowSession();
        const testSession = {
            callFlowTestDefinition: undefined,
            call: undefined,
            callFlowSession,
        };
        callServer.emit("testStarted", { testSession });
        jest.spyOn(callServer, "stop");
        callFlowSession.emit("timeoutWaitingForMatch", {
            transcription: "",
        });
        expect(callServer.stop).toHaveBeenCalled();
    });
    test("server not closed until all call-flow sessions have matched all their prompts", () => {
        const stopWhenAllTestsComplete = new CloseServerWhenTestsComplete_1.CloseServerWhenTestsComplete();
        stopWhenAllTestsComplete.initialise(pluginEmitter);
        pluginEmitter.emit("callServerStarted", {
            callServer: callServer,
        });
        const testSession1 = {
            callFlowTestDefinition: undefined,
            call: undefined,
            callFlowSession: new StubCallFlowSession(),
        };
        const testSession2 = {
            callFlowTestDefinition: undefined,
            call: undefined,
            callFlowSession: new StubCallFlowSession(),
        };
        callServer.emit("testStarted", { testSession: testSession1 });
        callServer.emit("testStarted", { testSession: testSession2 });
        jest.spyOn(callServer, "stop");
        testSession1.callFlowSession.emit("allPromptsMatched", {});
        expect(callServer.stop).not.toHaveBeenCalled();
    });
});
